from random import randint, seed, choice, sample
from string import ascii_uppercase as ABC, ascii_lowercase as abc
seed(47)

def save(fname, W, B):
    assert min(len(row) for row in B) == max(len(row) for row in B)
    f = open(fname, 'w')
    R, C = len(B), len(B[0])
    print(R, C, file=f)
    print(W, file=f)
    for row in B: print(row, file=f)
    f.close()

def gen_random(R, C, L, sigma):
    abeceda = list(sample(ABC, sigma))
    W = ''.join( choice(abeceda) for _ in range(L) )
    B = []
    for r in range(R):
        B.append( ''.join( choice(abeceda) for _ in range(C) ) )
    return (W, B)

def bias_letter(abeceda):
    if randint(0,99) < 90: return 'A'
    return choice(abeceda)

def gen_bias(R, C, L, sigma):
    abeceda = list(sample(ABC[1:], sigma))
    W = ''.join( bias_letter(abeceda) for _ in range(L) )
    B = []
    for r in range(R):
        B.append( ''.join( bias_letter(abeceda) for _ in range(C) ) )
    return (W, B)

def gen_smallcnt():
    R = randint(19, 30)
    C = randint(19, 30)
    good = list(sample(ABC,15))
    bad = [ x for x in ABC if x not in good ]
    pocty = [ 1 ] * 15
    for a in range(7):
        pocty[ randint(0,14) ] += 1
    B = [ [ choice(bad) for c in range(C) ] for r in range(R) ]
    for i in range(15):
        for j in range(pocty[i]):
            r = randint(0, R-1)
            c = randint(0, C-1)
            B[r][c] = good[i]
    W = ''.join(good)
    W += W
    B = [ ''.join(row) for row in B ]
    return (W, B)

save('00.sample.a.in', 'OI', ['KIVI','MALO','OBAL'])
save('00.sample.b.in', 'BRAT', ['BANAN','NEMAL','SUPKU'])
save('00.sample.c.in', 'UUUUUU', ['UUKUU','HESLO','UUPUU'])
save('00.sample.d.in', 'KSP', ['KSPPSKSP','PPSKSKSP','SKKSKSKP','PPPSKSKK','KKSKSKPP','PPPSKSKP'])

# V prvej sade vstupov (1 bod) platí $r, s, |w|\leq 10$.

save('01.a.in', 'A', ['A'])
save('01.b.in', 'A', ['B'])
save('01.c.in', 'AAA', ['A'])
save('01.d.in', 'AAA', ['AA','AA'])
save('01.e.in', 'AAA', ['AAA','AAA','AAA'])
save('01.f.in', 'AAA', ['AAAA','AAAA','AAAA','AAAA'])
save('01.g.in', *gen_random(5, 5, 5, 2))
save('01.h.in', *gen_random(5, 5, 5, 3))
for x in 'ijklmn':
    save(f'01.{x}.in', *gen_random(randint(3,5), randint(3,5), randint(3,5), randint(4,26)))

# V druhej sade vstupov (2 body) platí $r, s, |w|\leq 30$ a navyše v každom vstupe existuje dokopy 
# nanajvýš $10^6$ možností ako môže hyperjazdec preskákať nejaký prefix slova $w$.

save('02.a.in', 'MIFZBTVWROKAELPMIFZBTVWROKAELP', ['DJDUXDQYYQJNQNCNYQGYUGQCJ','JNCXSHXDJVUDQSPUQNUHYXSHQ','SNHQJUQQDNCCHJCDSNSHIXCJS','QNGNNNNHNJQCHJUCXGQDJSDCC','GJXGSUYQUYYDHSNQUXHHJJTGS','YDHUQSUQYHJJCQXQJYUUYYGCW','NDGJQGGGSHHUQQJDJJYSSCCHC','QCDUQYGSJSNQSSDXSAHJCCXQC','YYUZYUDGXCXSHNSNSNDYQSUJD','NCSXUUYNLSDYHSDHHXYJGSSHS','NJXUGSNHCUCJGSGSJUGYUYYSN','SYHDYHQUDHNYFCUUNDCXCMNUS','NGDHGCDHJXGGHDNNQQXQQHHQQ','PXGQYGYSHGSCGYYCCYYCHNYGN','QNHDBNGDNQDUHDJUJUGXNYJJQ','CJCXJQGJDNQSSUNVYNDDGNJOY','DYXDXCXSUSNUGGQXXNGQUHHCS','HUDUNDXCDHUVSQUYGMNZCUNQZ','GCXDRJUUDCQXDDCTYUDJDSSXY','NHSNHGNCDGCDJXSHSJUQYGJXE','QYJDXSNGNCNYUHCCCHUJXYJYV','YCHUSYHHQQYNNUJSJJJXYGYJH','UXUCJGUUNCSDCXDJHYYHHDDNG','UUYJGJQDQQJDECHUXGUSNUNNQ','CKCNXCCDQJSQQQJYUJXGJHHQN','SSSSNXHNCNCUJDDXQUSSUUJQS'])
for x in abc[1:20]:
    save(f'02.{x}.in', *gen_smallcnt())

# V tretej sade vstupov (2 body) platí $r, s, |w| \leq 40$.
# Vo štvrtej sade vstupov (2 body) platí $r, s, |w| \leq 100$.
# V piatej sade vstupov (3 body) platí $r, s\leq 800$ a $|w| \leq 100$.
RCDATA = [ None, None, None, 40, 100, 800 ]
LDATA = [ None, None, None, 40, 100, 100 ]

for batch in [3, 4, 5]:
    maxrc, maxl = RCDATA[batch], LDATA[batch]
    minrc, minl = int(0.9*maxrc), int(0.9*maxl)

    save(f'{batch:02d}.a.in', *gen_random(maxrc, maxrc, maxl, 1))
    save(f'{batch:02d}.b.in', *gen_random(maxrc, maxrc, maxl, 2))
    save(f'{batch:02d}.c.in', *gen_bias(maxrc, maxrc, maxl, 7))
    for x in abc[3:12]:
        save(f'{batch:02d}.{x}.in', *gen_random(randint(minrc,maxrc), randint(minrc,maxrc), randint(minl,maxl), randint(3,26)))

